@@ -16,7 +16,7 @@
 ** with lots of bits pasted from libpng.txt by Guy Eric Schalnat
 */
 
-#define VERSION "2.37.3 (13 July 1999)"
+/* GRR 19991203:  moved VERSION to new version.h header file */
 
 /* GRR 19990713:  fixed redundant freeing of png_ptr and info_ptr in setjmp()
  *  blocks and added "pm_close(ifp)" in each.  */
@@ -29,11 +29,20 @@
 #include <math.h>
 #include "pnm.h"
 #include "png.h"	/* includes zlib.h and setjmp.h */
+#include "version.h"	/* VERSION macro */
 
 typedef struct _jmpbuf_wrapper {
   jmp_buf jmpbuf;
 } jmpbuf_wrapper;
 
+/* GRR 19991205:  this is used as a test for pre-1999 versions of netpbm and
+ *   pbmplus vs. 1999 or later (in which pm_close was split into two)
+ */
+#ifdef PBMPLUS_RAWBITS
+#  define pm_closer pm_close
+#  define pm_closew pm_close
+#endif
+
 #ifndef TRUE
 #  define TRUE 1
 #endif
@@ -264,31 +273,31 @@ FILE *tfp;
   type_string = alpha_string = "";
 
   if (fread (sig_buf, 1, SIG_CHECK_SIZE, ifp) != SIG_CHECK_SIZE) {
-    pm_close (ifp);
+    pm_closer (ifp);
     pm_error ("input file empty or too short");
   }
   if (png_sig_cmp (sig_buf, (png_size_t) 0, (png_size_t) SIG_CHECK_SIZE) != 0) {
-    pm_close (ifp);
+    pm_closer (ifp);
     pm_error ("input file not a PNG file");
   }
 
   png_ptr = png_create_read_struct (PNG_LIBPNG_VER_STRING,
     &pngtopnm_jmpbuf_struct, pngtopnm_error_handler, NULL);
   if (png_ptr == NULL) {
-    pm_close (ifp);
+    pm_closer (ifp);
     pm_error ("cannot allocate LIBPNG structure");
   }
 
   info_ptr = png_create_info_struct (png_ptr);
   if (info_ptr == NULL) {
     png_destroy_read_struct (&png_ptr, (png_infopp)NULL, (png_infopp)NULL);
-    pm_close (ifp);
+    pm_closer (ifp);
     pm_error ("cannot allocate LIBPNG structures");
   }
 
   if (setjmp (pngtopnm_jmpbuf_struct.jmpbuf)) {
     png_destroy_read_struct (&png_ptr, &info_ptr, (png_infopp)NULL);
-    pm_close (ifp);
+    pm_closer (ifp);
     pm_error ("setjmp returns error condition");
   }
 
@@ -344,7 +353,7 @@ FILE *tfp;
   png_image = (png_byte **)malloc (info_ptr->height * sizeof (png_byte*));
   if (png_image == NULL) {
     png_destroy_read_struct (&png_ptr, &info_ptr, (png_infopp)NULL);
-    pm_close (ifp);
+    pm_closer (ifp);
     pm_error ("couldn't allocate space for image");
   }
 
@@ -369,7 +378,7 @@ FILE *tfp;
         free (png_image[x]);
       free (png_image);
       png_destroy_read_struct (&png_ptr, &info_ptr, (png_infopp)NULL);
-      pm_close (ifp);
+      pm_closer (ifp);
       pm_error ("couldn't allocate space for image");
     }
   }
@@ -560,7 +569,7 @@ FILE *tfp;
       free (png_image[y]);
     free (png_image);
     png_destroy_read_struct (&png_ptr, &info_ptr, (png_infopp)NULL);
-    pm_close (ifp);
+    pm_closer (ifp);
     pm_error ("couldn't allocate space for image");
   }
 
@@ -674,7 +683,7 @@ FILE *tfp;
             free (png_image[i]);
           free (png_image);
           png_destroy_read_struct (&png_ptr, &info_ptr, (png_infopp)NULL);
-          pm_close (ifp);
+          pm_closer (ifp);
           pm_error ("unknown PNG color type");
       }
       pnm_pixel++;
@@ -764,9 +773,9 @@ char *argv[];
   convertpng (ifp, tfp);
 
   if (text)
-    pm_close (tfp);
+    pm_closew (tfp);
 
-  pm_close (ifp);
-  pm_close (stdout);
+  pm_closer (ifp);
+  pm_closew (stdout);
   exit (0);
 }
