@@ -17,7 +17,10 @@
 ** implied warranty.
 */
 
-#define VERSION "2.37.3 (13 July 1999)"
+/* GRR 19991203:  incorporated fix by Rafal Rzeczkowski <rzeczkrg@mcmaster.ca>:
+ *  gray images with exactly 16 shades were being to 8-bit grayscale rather
+ *  than 4-bit palette due to misuse of the pm_maxvaltobits() function.  Also
+ *  moved VERSION to new version.h header file. */
 
 /* GRR 19990713:  fixed redundant freeing of png_ptr and info_ptr in setjmp()
  *  blocks and added png_destroy_write_struct() and pm_close(ifp) in each
@@ -46,6 +49,7 @@
 
 #include "pnm.h"
 #include "png.h"	/* includes zlib.h and setjmp.h */
+#include "version.h"	/* VERSION macro */
 
 #include "ppmcmap.h"
 
@@ -53,6 +57,14 @@ typedef struct _jmpbuf_wrapper {
   jmp_buf jmpbuf;
 } jmpbuf_wrapper;
 
+/* GRR 19991205:  this is used as a test for pre-1999 versions of netpbm and
+ *   pbmplus vs. 1999 or later (in which pm_close was split into two) 
+ */
+#ifdef PBMPLUS_RAWBITS
+#  define pm_closer pm_close
+#  define pm_closew pm_close
+#endif
+
 #ifndef TRUE
 #  define TRUE 1
 #endif
@@ -311,20 +323,20 @@ FILE *tfp;
   png_ptr = png_create_write_struct (PNG_LIBPNG_VER_STRING,
     &pnmtopng_jmpbuf_struct, pnmtopng_error_handler, NULL);
   if (png_ptr == NULL) {
-    pm_close (ifp);
+    pm_closer (ifp);
     pm_error ("cannot allocate LIBPNG structure");
   }
 
   info_ptr = png_create_info_struct (png_ptr);
   if (info_ptr == NULL) {
     png_destroy_write_struct (&png_ptr, (png_infopp)NULL);
-    pm_close (ifp);
+    pm_closer (ifp);
     pm_error ("cannot allocate LIBPNG structures");
   }
 
   if (setjmp (pnmtopng_jmpbuf_struct.jmpbuf)) {
     png_destroy_write_struct (&png_ptr, &info_ptr);
-    pm_close (ifp);
+    pm_closer (ifp);
     pm_error ("setjmp returns error condition (1)");
   }
 
@@ -352,7 +364,7 @@ FILE *tfp;
     alpha_mask = pgm_readpgm (afp, &alpha_cols, &alpha_rows, &alpha_maxval);
     if (alpha_cols != cols || alpha_rows != rows) {
       png_destroy_write_struct (&png_ptr, &info_ptr);
-      pm_close (ifp);
+      pm_closer (ifp);
       pm_error ("dimensions for image and alpha mask do not agree");
     }
     /* check if the alpha mask can be represented by a single transparency
@@ -429,7 +441,7 @@ FILE *tfp;
   if (pnm_type != PBM_TYPE || alpha) {
     if (maxval > 65535 && !downscale) {
       png_destroy_write_struct (&png_ptr, &info_ptr);
-      pm_close (ifp);
+      pm_closer (ifp);
       pm_error ("can only handle files up to 16-bit (use -downscale to override");
     }
 
@@ -642,7 +654,7 @@ FILE *tfp;
        - for ppm files if we have <= 256 colors
        - for alpha if we have <= 256 color/transparency pairs
        - for pgm files (with or without alpha) if the number of bits needed for
-         the gray-transparency pairs is smaller then the number of bits needed
+         the gray-transparency pairs is smaller than the number of bits needed
          for maxval
      When maxval > 255, we never write a paletted image.
   */
@@ -665,8 +677,9 @@ FILE *tfp;
     if (verbose) {
       pm_message ("%d colors found", colors);
     }
-    if ((chv == (colorhist_vector) NULL) ||    /* GRR:    v--- was `=' here */
-        (pnm_type == PGM_TYPE && pm_maxvaltobits(colors) > 
+    /* RR:  note that if colors == 16, maxval == 15 (i.e., range == 0-15) */
+    if ((chv == (colorhist_vector) NULL) ||    /* GRR:      v-- was `=' here */
+        (pnm_type == PGM_TYPE && pm_maxvaltobits(colors-1) > 
                                 (pm_maxvaltobits(maxval) / 2)) ||
         (pnm_type == PPM_TYPE && maxval > 255)) {
       chv = NULL;
@@ -687,7 +700,7 @@ FILE *tfp;
               == NULL)
           {
             png_destroy_write_struct (&png_ptr, &info_ptr);
-            pm_close (ifp);
+            pm_closer (ifp);
             pm_error ("out of memory allocating alpha/palette entries");
           }
           alphas_of_color_cnt[i] = 0;
@@ -802,7 +815,7 @@ FILE *tfp;
       depth = 1;
     else {
       png_destroy_write_struct (&png_ptr, &info_ptr);
-      pm_close (ifp);
+      pm_closer (ifp);
       pm_error (" (can't happen) undefined maxval");
     }
 
@@ -830,7 +843,7 @@ FILE *tfp;
 
   if (setjmp (pnmtopng_jmpbuf_struct.jmpbuf)) {
     png_destroy_write_struct (&png_ptr, &info_ptr);
-    pm_close (ifp);
+    pm_closer (ifp);
     pm_error ("setjmp returns error condition (2)");
   }
 
@@ -927,7 +940,7 @@ FILE *tfp;
       /* indices should have just crossed paths */
       if (bot_idx != top_idx + 1) {
         png_destroy_write_struct (&png_ptr, &info_ptr);
-        pm_close (ifp);
+        pm_closer (ifp);
         pm_error ("internal inconsistency: remapped bot_idx = %d, top_idx = %d",
           bot_idx, top_idx);
       }
@@ -982,7 +995,7 @@ FILE *tfp;
     } else {
       if (transparent > 0) {
         png_destroy_write_struct (&png_ptr, &info_ptr);
-        pm_close (ifp);
+        pm_closer (ifp);
         pm_error (" (can't happen) transparency AND alpha");
       }
     }
@@ -1080,7 +1093,7 @@ FILE *tfp;
   if ((line = (png_byte *) malloc (cols*8)) == NULL)
   {
     png_destroy_write_struct (&png_ptr, &info_ptr);
-    pm_close (ifp);
+    pm_closer (ifp);
     pm_error ("out of memory allocating PNG row buffer");
   }
 
@@ -1120,7 +1133,7 @@ FILE *tfp;
           *pp++ = PPM_GETB (p)&0xff;
         } else {
           png_destroy_write_struct (&png_ptr, &info_ptr);
-          pm_close (ifp);
+          pm_closer (ifp);
           pm_error (" (can't happen) undefined color_type");
         }
         if (info_ptr->color_type & PNG_COLOR_MASK_ALPHA) {
@@ -1352,12 +1365,12 @@ char *argv[];
   convertpnm (ifp, afp, tfp);
 
   if (alpha)
-    pm_close (afp);
+    pm_closer (afp);
   if ((text) || (ztxt))
-    pm_close (tfp);
+    pm_closer (tfp);
 
-  pm_close (ifp);
-  pm_close (stdout);
+  pm_closer (ifp);
+  pm_closew (stdout);
 
   exit (0);
 }
